/*
   3D Invaders

   bombs.cc

   (c)2002 Paul Boese
*/

#include "common.h"
#include "bombs.h"

enum { B_LOADED, B_DROP, B_DROPPING, B_HIT, B_MISS, B_RELOAD };

int Bombs::refcnt=0;

Bombs::Bombs() {
	velocity = -1.0f;
	smode = B_LOADED;
	rotf = 0.0f;
	roti = 0;
	refcnt++;
}

void Bombs::Drop(point_t * initial_pos) {
	if (smode == B_LOADED) {
		position.x = initial_pos->x;
		position.y = initial_pos->y;
		position.z = initial_pos->z;
		velocity = 0;
		smode = B_DROP;
	} 
}

void Bombs::Dropped() {
	switch (smode) {
	case B_LOADED:
		break;
	case B_DROP:
		smode = B_DROPPING;
		// fire sound effect
		snd_sfx_play(sfx_bomb_drop, gbl_sfx_volume, 128); 
		break;
	case B_DROPPING:
		smode = draw_flight();
		break;
	case B_HIT:
	case B_MISS:
		smode = B_RELOAD;
		break;
	case B_RELOAD:
		smode = B_LOADED;
		break;
	default:
		assert_msg(0, "Unknown Bombs mode");
	}
}

int Bombs::Active() {
	return (smode == B_DROPPING)? -1 : 0;
}

void Bombs::Hit() {
	smode = B_HIT;
}

int Bombs::draw_flight() {
	// draw missle object
	static float frame[16][2] = {
		{0.75f, 0.0f}, {0.75f, 0.25f}, {0.75f, 0.5f}, {0.75f, 0.75f},
		{0.5f,  0.0f}, {0.5f,  0.25f}, {0.5f,  0.5f}, {0.5f,  0.75f},
		{0.25f, 0.0f}, {0.25f, 0.25f}, {0.25f, 0.5f}, {0.25f, 0.75f},
		{0.0f,  0.0f}, {0.0f,  0.25f}, {0.0f,  0.5f}, {0.0f,  0.75f}
	};
	GLint idx=(roti++)%16;
	GLfloat u=frame[idx][0], v=frame[idx][1];
	glEnable(GL_TEXTURE_2D);
	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glBindTexture(GL_TEXTURE_2D, txbombs);
	glBegin(GL_QUADS);
	glColor3f(1.0, 1.0, 1.0);
	glTexCoord2f(u,       v      ); glVertex3f(-1.0, -1.0, 0.0);
	glTexCoord2f(u+0.25f, v      ); glVertex3f( 1.0, -1.0, 0.0);
	glTexCoord2f(u+0.25f, v+0.25f); glVertex3f( 1.0,  1.0, 0.0);
	glTexCoord2f(u,       v+0.25f); glVertex3f(-1.0,  1.0, 0.0);
	glEnd();
	glPopMatrix();
	// calculate next position
	position.z += 1.0f; 
	if (position.z < 20.0f) {
		return B_DROPPING;
	}
	else {
		return B_MISS;
	}
}
