/* 
   3D Invaders

   gfx.cpp

   (c)2002 Paul Boese
*/

#include "common.h"
#include <zlib/zlib.h>
#include <pcx/pcx.h>
#include <png/png.h>

pvr_init_params_t params = {
	/* Enable opaque, translucent and punchthru polygons with size 16 */
	{ PVR_BINSIZE_16, PVR_BINSIZE_0, PVR_BINSIZE_16, PVR_BINSIZE_0, PVR_BINSIZE_0 },

	/* Vertex buffer size 1024K */
	1024*1024
};

int gfx_init() {
	/* Initialize PVR */
	if (pvr_init(&params) < 0)
		return -1;

	/* Initialize GL */
	glKosInit();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	//gluPerspective(45.0f, 640.0f / 480.0f, 0.1f, 100.0f);
	gluPerspective(55.0f, 640.0f / 480.0f, 0.1f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
	glEnable(GL_TEXTURE_2D);
	glShadeModel(GL_SMOOTH);

	/* Expect CCW verts */
	glFrontFace(GL_CCW);

        /* Enable Transparancy */
        //glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_DEPTH_TEST);


	/* Load Textures */
	gfx_load_png("/rd/shots.png", &txshots, 1);
	gfx_load_png("/rd/bombs.png", &txbombs, 1);
	gfx_load_png("/rd/invaders.png", &txinvaders, 0);
	gfx_load_png("/rd/0121w.png", &tx0121w, 0);
	gfx_load_png("/rd/ship.png", &txship, 1);
	
	return 0;
}

void gfx_shutdown() {
	pvr_stats_t	stats;

	pvr_get_stats(&stats);
	dbglog(DBG_DEBUG, "3D Stats: %d frames, frame rate ~%f fps\n",
		stats.frame_count, stats.frame_rate);
}

void gfx_load_pcx(const char *fn, GLuint *txr) {
	kos_img_t img;
	pvr_ptr_t txaddr;

	if (pcx_to_img(fn, &img) < 0) {
		printf("can't load %s\n", fn);
		return;
	}

	txaddr = pvr_mem_malloc(img.w * img.h * 2);
	pvr_txr_load_kimg(&img, txaddr, PVR_TXRLOAD_INVERT_Y);
	kos_img_free(&img, 0);

	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	glKosTex2D(GL_RGB565_TWID, img.w, img.h, txaddr);
}

void gfx_load_png(char *fn, GLuint *txr, GLuint use_alpha) {
	kos_img_t img;
	pvr_ptr_t txaddr;
	GLuint mask = use_alpha ? PNG_FULL_ALPHA : PNG_NO_ALPHA;
	GLuint frmt = use_alpha ? GL_ARGB4444_TWID : GL_RGB565_TWID;

	if (png_to_img(fn, mask, &img) < 0) {
		printf("Can't load '%s'\n", fn);
		return;
	}

	txaddr = pvr_mem_malloc(img.w * img.h * 2);
	pvr_txr_load_kimg(&img, txaddr, PVR_TXRLOAD_INVERT_Y);
	kos_img_free(&img, 0);

	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	glKosTex2D(frmt, img.w, img.h, txaddr);
}
